==> fastec_3.4.9.tar.xz: usr/local/include/fastec/fi_stream_avi.h <==
/*!
 * @file fas_stream_avi.h
 * @brief The AVI stream object of the Fastec Video library.
 *
 * Copyright 2019 (c) Fastec Imaging as an unpublished work.
 * All Rights Reserved.
 *
 * The information contained herein is confidential property of
 * Fastec Imaging. The use, copying, transfer or disclosure of such
 * information is prohibited except by express written agreement
 * with Fastec Imaging.
 */

#ifndef FASTEC_AVI_STREAM_H_
#define FASTEC_AVI_STREAM_H_
#pragma once

//--------------------------------------------------------------------
// Includes
//--------------------------------------------------------------------

#include <stdint.h>
#include <memory>
#include <mutex>
#include <vector>
#include <deque>

#include "fas_video.h"
#include "fi_stream.h"

#ifdef MAC_OS
typedef off_t off64_t;
#endif

namespace FAS_AVI
{
//--------------------------------------------------------------------
// Constant Definitions
//--------------------------------------------------------------------

typedef enum AVI_SEEK_POS
{
    AVI_SEEK_SET,
    AVI_SEEK_CUR,
    AVI_SEEK_END
} AVI_SEEK_POS;

typedef std::vector<uint8_t> PLACE_HOLDER_FRAME;

//--------------------------------------------------------------------
// Forward Declarations
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// Class Declaration
//--------------------------------------------------------------------

class FiVideoAVIStream : public FAS_VIDEO::FiVideoStream
{
public:
    FiVideoAVIStream();
    virtual ~FiVideoAVIStream();

    // This object, for now, is non-copyable and non-copy assignable.
    FiVideoAVIStream(const FiVideoAVIStream& i) = delete;
    FiVideoAVIStream& operator=(const FiVideoAVIStream& i) = delete;

    virtual FAS_VIDEO::STREAM_STATUS PrepareForReading(const std::string& videoName) override;

    virtual FAS_VIDEO::STREAM_STATUS PrepareForWriting(const std::string& videoName,
                                                       uint32_t frameRate,
                                                       uint32_t frameWidth,
                                                       uint32_t frameHeight,
                                                       uint32_t frameCount) override;

    virtual FAS_VIDEO::STREAM_STATUS AddVideoFrame(std::shared_ptr<FAS_VIDEO::FiImage> imageDataFrame) override;

    virtual FAS_VIDEO::STREAM_STATUS AddPlaceholderFrame() override;

    virtual FAS_VIDEO::STREAM_STATUS EditVideoFrame(uint32_t frame_num, std::shared_ptr<FAS_VIDEO::FiImage> imageDataFrame) override;

    virtual FAS_VIDEO::STREAM_STATUS IsStreamWriterComplete() override;

    virtual std::unique_ptr<FAS_VIDEO::FiImage> LoadVideoFrame(uint32_t frameNum) override;

    virtual FAS_VIDEO::STREAM_STATUS CloseStream() override;

protected:
    //! The standard library's std::fstream AVI File I/O object.
    std::unique_ptr<std::fstream> ioFileStreamPtr;

    //! Keeps track of the header processing for writer.
    bool headerProcessed;

    //! Keeps track of lost frame(s) condition at the very beginning of the video.
    bool lostFramesInFront;

    //! Keeps track of how many lost frame(s) were processed for beginning of the video.
    int lostFramesInFrontCount;

    //! Keeps track of the file offset as AVI stream is manipulated.
    uint64_t curFileOffset;

    //! Called internally when the stream has enough information to write the AVI header.
    //! This occurs when the first AddVideoFrame is invoked and headerProcessed is false.
    bool InitializeHeader();

    //! Called internally when preparing the AVI stream reader to validate the contents
    //! of the AVI header information.
    bool ScanHeader();

    //! Called internally when scanning the AVI header information to validate AVI
    //! index record.
    bool ScanIndex();

    //! Common test to determine if the AVI stream is in a valid state for reading.
    bool OkToRead(const char *whoIsAsking, bool addContextCheck);

    //! Common test to determine if the AVI stream is in a valid state for writing.
    bool OkToWrite(const char *whoIsAsking, bool addContextCheck, bool addImageAttachCheck);

    //! Called internally when reading data from file i/o stream.
    bool ReadData(void *buffer, uint32_t length);

    //! Called internally when writing data to file i/o stream.
    bool WriteData(const void *buffer, uint32_t length);

    //! Called internally when setting the seek position in file output stream.
    bool SetSeekPos(off64_t seekPos, FAS_AVI::AVI_SEEK_POS howToSeek);

    //! Called internally when the current seek position is requested.
    off64_t GetSeekPos();

    //! Called internally when the object is destructing or when an error has occurred.
    bool AVICleanup();

    //! Recursive support for reading AVI list information from header.
    uint32_t AVIReadList(uint32_t sizeToRead);

    //! Reads a chunk of index information from AVI header file.
    uint32_t AVIReadChunk(uint32_t fccId, uint32_t sizeToRead);

    //! Called internally when the header is being created or when a new RIFF is needed.
    bool AVIStartRiff();

    //! Called internally when done with a video riff.
    bool AVIEndRiff();

    //! Called by the AddFrame and AddPlaceholderFrame functions to facilitate adding
    //! image data to an existing AVI Video Stream.
    bool AVIAddFrame(bool addPlaceholderFrame);

    //! Provides ability to locate where in the file a frame needs to be written
    //! in the case of frames that were lost and are now being inserted.
    off64_t AVILookupImageOffset(uint32_t frame_num, size_t *size);

    //! The port of the old AVI writer requires many structures and constants
    //! that don't need to be part of the outside world beyond this AVI streaming
    //! object.  Therefore a PImpl class design will hide these implementation
    //! details in the following internal class.
    class AVIContextClass;
    std::unique_ptr<AVIContextClass> aviContext;

    //! A list of placeholder frames if any.
    mutable std::mutex placeHolderFramesMutex;
    std::deque<uint32_t> placeHolderFrames;
};

} // End of FAS_AVI Namespace

#endif /* !FASTEC_AVI_STREAM_H_ */

==> fastec_3.5.0_12.tar.xz: usr/local/include/fastec/fi_stream_avi.h <==
/*!
 * @file fas_stream_avi.h
 * @brief The AVI stream object of the Fastec Video library.
 *
 * Copyright 2019 (c) Fastec Imaging as an unpublished work.
 * All Rights Reserved.
 *
 * The information contained herein is confidential property of
 * Fastec Imaging. The use, copying, transfer or disclosure of such
 * information is prohibited except by express written agreement
 * with Fastec Imaging.
 */

#ifndef FASTEC_AVI_STREAM_H_
#define FASTEC_AVI_STREAM_H_
#pragma once

//--------------------------------------------------------------------
// Includes
//--------------------------------------------------------------------

#include <stdint.h>
#include <memory>
#include <mutex>
#include <vector>
#include <deque>

#include "fas_video.h"
#include "fi_stream.h"

#ifdef MAC_OS
typedef off_t off64_t;
#endif

namespace FAS_AVI
{
//--------------------------------------------------------------------
// Constant Definitions
//--------------------------------------------------------------------

typedef enum AVI_SEEK_POS
{
    AVI_SEEK_SET,
    AVI_SEEK_CUR,
    AVI_SEEK_END
} AVI_SEEK_POS;

typedef std::vector<uint8_t> PLACE_HOLDER_FRAME;

//--------------------------------------------------------------------
// Forward Declarations
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// Class Declaration
//--------------------------------------------------------------------

class FiVideoAVIStream : public FAS_VIDEO::FiVideoStream
{
public:
    FiVideoAVIStream();
    virtual ~FiVideoAVIStream();

    // This object, for now, is non-copyable and non-copy assignable.
    FiVideoAVIStream(const FiVideoAVIStream& i) = delete;
    FiVideoAVIStream& operator=(const FiVideoAVIStream& i) = delete;

    virtual FAS_VIDEO::STREAM_STATUS PrepareForReading(const std::string& videoName) override;

    virtual FAS_VIDEO::STREAM_STATUS PrepareForWriting(const std::string& videoName,
                                                       uint32_t frameRate,
                                                       uint32_t frameWidth,
                                                       uint32_t frameHeight,
                                                       uint32_t frameCount) override;

    virtual FAS_VIDEO::STREAM_STATUS AddVideoFrame(std::shared_ptr<FAS_VIDEO::FiImage> imageDataFrame) override;

    virtual FAS_VIDEO::STREAM_STATUS AddPlaceholderFrame() override;

    virtual FAS_VIDEO::STREAM_STATUS EditVideoFrame(uint32_t frame_num, std::shared_ptr<FAS_VIDEO::FiImage> imageDataFrame) override;

    virtual FAS_VIDEO::STREAM_STATUS IsStreamWriterComplete() override;

    virtual std::unique_ptr<FAS_VIDEO::FiImage> LoadVideoFrame(uint32_t frameNum) override;

    virtual FAS_VIDEO::STREAM_STATUS CloseStream() override;

protected:
    //! The standard library's std::fstream AVI File I/O object.
    std::unique_ptr<std::fstream> ioFileStreamPtr;

    //! Keeps track of the header processing for writer.
    bool headerProcessed;

    //! Keeps track of frames added to overall stream either through AddVideoFrame or AddPlaceholderFrame.
    uint32_t runningFrameCount;

    //! Keeps track of the file offset as AVI stream is manipulated.
    uint64_t curFileOffset;

    //! Does the real work of adding a frame to the AVI stream.
    bool InsertPlaceholderFrame();

    //! Called internally when the stream has enough information to write the AVI header.
    //! This occurs when the first AddVideoFrame is invoked and headerProcessed is false.
    bool InitializeHeader();

    //! Called internally when preparing the AVI stream reader to validate the contents
    //! of the AVI header information.
    bool ScanHeader();

    //! Called internally when scanning the AVI header information to validate AVI
    //! index record.
    bool ScanIndex();

    //! Common test to determine if the AVI stream is in a valid state for reading.
    bool OkToRead(const char *whoIsAsking, bool addContextCheck);

    //! Common test to determine if the AVI stream is in a valid state for writing.
    bool OkToWrite(const char *whoIsAsking, bool addContextCheck, bool addImageAttachCheck);

    //! Called internally when reading data from file i/o stream.
    bool ReadData(void *buffer, uint32_t length);

    //! Called internally when writing data to file i/o stream.
    bool WriteData(const void *buffer, uint32_t length);

    //! Called internally when setting the seek position in file output stream.
    bool SetSeekPos(off64_t seekPos, FAS_AVI::AVI_SEEK_POS howToSeek);

    //! Called internally when the current seek position is requested.
    off64_t GetSeekPos();

    //! Called internally when the object is destructing or when an error has occurred.
    bool AVICleanup();

    //! Recursive support for reading AVI list information from header.
    uint32_t AVIReadList(uint32_t sizeToRead);

    //! Reads a chunk of index information from AVI header file.
    uint32_t AVIReadChunk(uint32_t fccId, uint32_t sizeToRead);

    //! Called internally when the header is being created or when a new RIFF is needed.
    bool AVIStartRiff();

    //! Called internally when done with a video riff.
    bool AVIEndRiff();

    //! Called by the AddFrame and AddPlaceholderFrame functions to facilitate adding
    //! image data to an existing AVI Video Stream.
    bool AVIAddFrame(bool addPlaceholderFrame);

    //! Provides ability to locate where in the file a frame needs to be written
    //! in the case of frames that were lost and are now being inserted.
    off64_t AVILookupImageOffset(uint32_t frame_num, size_t *size);

    //! The port of the old AVI writer requires many structures and constants
    //! that don't need to be part of the outside world beyond this AVI streaming
    //! object.  Therefore a PImpl class design will hide these implementation
    //! details in the following internal class.
    class AVIContextClass;
    std::unique_ptr<AVIContextClass> aviContext;

    //! A list of placeholder frames if any.
    mutable std::mutex placeHolderFramesMutex;
    std::deque<uint32_t> placeHolderFrames;
};

} // End of FAS_AVI Namespace

#endif /* !FASTEC_AVI_STREAM_H_ */
